#include <signal.h>
#include <msp/io/print.h>
#include "application.h"
#include "getopt.h"

using namespace std;

namespace Msp {

Application *Application::app_ = 0;
Application::Starter *Application::starter_ = 0;
void *Application::data_ = 0;

Application::Application():
	exit_code(0)
{ }

int Application::run(int argc, char **argv, void *data)
{
	static bool called = false;
	if(called)
	{
		IO::cerr.write("Trying to call Application::run_app twice!\n");
		return 125;
	}
	called = true;

	if(!starter_)
	{
		IO::cerr.write("Trying to run with no RegisteredApplication class!\n");
		return 126;
	}

	data_ = data;

	try
	{
		try
		{
			app_ = starter_->create_app(argc, argv);
		}
		catch(const usage_error &e)
		{
			IO::print(IO::cerr, "%s\n%s\n", e.what(), e.help());
			return 1;
		}

		int result = app_->main();
		Application *a = app_;
		app_ = 0;
		delete a;
		return result;
	}
	catch(const exception &e)
	{
		delete app_;

		display_exception(e);

		return 124;
	}
}

int Application::main()
{
	done = false;
	while(!done)
		tick();

	return exit_code;
}

void Application::catch_signal(int s)
{
	signal(s, &sighandler_);
}

void Application::exit(int c)
{
	done = true;
	exit_code = c;
}

void Application::sighandler_(int s)
{
	app_->sighandler(s);
}


Application::Starter::Starter()
{
	if(starter_)
		throw logic_error("Can't create more than one Starter instance");

	starter_ = this;
}

} // namespace Msp
